import 'dart:async';

import 'package:camera/camera.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';

import 'index.dart';

enum MaskType { idCard, passport, selfie }

// A widget that allows users to take a picture using a given camera.
class OCRCameraWidget extends StatefulWidget {
  final List<CameraDescription> camera;
  final String title;
  final TextStyle tStyle;
  final String firstDescription;
  final TextStyle fdStyle;
  final String secondDescription;
  final TextStyle sdStyle;
  final String questionToConfirmUseByTakenPhoto;
  final TextStyle qtcubtpStyle;
  final String rejectText;
  final TextStyle rtStyle;
  final String confirmText;
  final TextStyle cStyle;
  final MaskType maskType;
  final Widget takePhotoButton;
  final ValueChanged<XFile?> onResult;

  static const _defaultTStyle = TextStyle(
    fontSize: 14,
    fontWeight: FontWeight.w500,
    color: Colors.white,
  );
  static const _defaultFdStyle = TextStyle(
    fontWeight: FontWeight.w400,
    fontSize: 16,
  );

  static const _defaultSdStyle = TextStyle(
    fontWeight: FontWeight.w400,
    fontSize: 12,
    color: Color(0xffC7C7CC),
  );

  static const _defaultQTCUBTPStyle = TextStyle(
    fontSize: 12,
    fontWeight: FontWeight.w400,
    color: Color(0xffC7C7CC),
  );

  static const _defaultRtStyle = TextStyle(
    fontWeight: FontWeight.w400,
    fontSize: 18,
  );
  static const _defaultCStyle = TextStyle(
    fontWeight: FontWeight.w400,
    fontSize: 18,
    color: Colors.black,
  );

  static const _takePhotoButton = TakePhotoButton();

  const OCRCameraWidget({
    Key? key,
    required this.camera,
    this.maskType = MaskType.idCard,
    required this.onResult,
    this.title = 'This is title',
    this.tStyle = _defaultTStyle,
    this.firstDescription = 'This is first description',
    this.fdStyle = _defaultFdStyle,
    this.secondDescription = 'This is second description',
    this.sdStyle = _defaultSdStyle,
    this.questionToConfirmUseByTakenPhoto =
        'This is question to confirm use by taken photo',
    this.qtcubtpStyle = _defaultQTCUBTPStyle,
    this.rejectText = 'Captures',
    this.rtStyle = _defaultRtStyle,
    this.confirmText = 'Using this photo',
    this.cStyle = _defaultCStyle,
    this.takePhotoButton = _takePhotoButton,
  }) : super(key: key);

  @override
  OCRCameraWidgetState createState() => OCRCameraWidgetState();
}

class OCRCameraWidgetState extends State<OCRCameraWidget> {
  late CameraController _controller;
  late Future<void> _initializeControllerFuture;
  late MaskType _maskType;
  String _imagePath = '';
  bool _isShowIDCardMask = true;

  @override
  void initState() {
    super.initState();

    _maskType = widget.maskType;
    _isShowIDCardMask = _maskType == MaskType.idCard ? true : false;

    int positionCamera = _maskType == MaskType.selfie ? 1 : 0;

    // To display the current output from the Camera,
    // create a CameraController.
    _controller = CameraController(
      // Get a specific camera from the list of available cameras.
      widget.camera[positionCamera],
      // Define the resolution to use.
      ResolutionPreset.max,
    );

    // Next, initialize the controller. This returns a Future.
    _initializeControllerFuture = _controller.initialize();
  }

  @override
  void dispose() {
    // Dispose of the controller when the widget is disposed.
    _controller.dispose();
    _closeStream();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Size _size = MediaQuery.of(context).size;
    double _paddingTop = MediaQuery.of(context).padding.top;
    return Scaffold(
      // You must wait until the controller is initialized before displaying the
      // camera preview. Use a FutureBuilder to display a loading spinner until the
      // controller has finished initializing.
      body: SafeArea(
        child: Stack(
          fit: StackFit.expand,
          children: [
            FutureBuilder<void>(
              future: _initializeControllerFuture,
              builder: (context, snapshot) {
                if (snapshot.connectionState == ConnectionState.done) {
                  // If the Future is complete, display the preview.
                  return CameraPreview(_controller);
                  // return CircularProgressIndicator();
                } else {
                  // Otherwise, display a loading indicator.
                  return const Center(child: CircularProgressIndicator());
                }
              },
            ),
            OcrPreview(cameraStream: _cameraStream),
            Container(
              constraints: const BoxConstraints.expand(),
              child: Container(
                color: Colors.transparent,
                width: _size.width,
                height: _size.height,
                child: Stack(
                  fit: StackFit.expand,
                  children: [
                    MaskPreview(
                      cameraStream: _cameraStream,
                      maskType: _maskType,
                    ),
                    Positioned(
                      top: 8,
                      child: OcrTitle(size: _size, widget: widget),
                    ),
                    Positioned(
                      top: (_size.height - _paddingTop) / 2 -
                          _size.width *
                              0.9 /
                              3 *
                              (_isShowIDCardMask ? 2 : 4) /
                              2 -
                          30,
                      left: 16,
                      right: 16,
                      child: OcrFirstDescription(widget: widget),
                    ),
                    Positioned(
                      top: (_size.height - _paddingTop) / 2 +
                          _size.width *
                              0.9 /
                              3 *
                              (_isShowIDCardMask ? 2 : 4) /
                              2 +
                          12,
                      left: 16,
                      right: 16,
                      child: OcrSecondDescription(widget: widget),
                    ),
                    ContainerMask(
                        initializeControllerFuture: _initializeControllerFuture,
                        cameraController: _controller,
                        cameraStream: _cameraStream,
                        cameraSink: _cameraSink,
                        widget: widget),
                  ],
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  final StreamController<String> _cameraStreamController =
      StreamController<String>.broadcast();

  Sink<String> get _cameraSink => _cameraStreamController.sink;

  Stream<String> get _cameraStream => _cameraStreamController.stream;

  void _closeStream() {
    _cameraStreamController.close();
  }
}
